# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from scaleway_core.bridge import (
    unmarshal_Money,
    unmarshal_TimeSeries,
)
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from dateutil import parser
from .types import (
    ServerBootType,
    BMCAccess,
    CPU,
    CreateServerRequestInstall,
    Disk,
    GetServerMetricsResponse,
    IP,
    ListOSResponse,
    ListOffersResponse,
    ListOptionsResponse,
    ListServerEventsResponse,
    ListServerPrivateNetworksResponse,
    ListServersResponse,
    ListSettingsResponse,
    Memory,
    OS,
    OSOSField,
    Offer,
    OfferOptionOffer,
    Option,
    PersistentMemory,
    RaidController,
    Server,
    ServerEvent,
    ServerInstall,
    ServerOption,
    ServerPrivateNetwork,
    ServerRescueServer,
    SetServerPrivateNetworksResponse,
    Setting,
    CreateServerRequest,
    UpdateServerRequest,
    InstallServerRequest,
    RebootServerRequest,
    StartServerRequest,
    StartBMCAccessRequest,
    UpdateIPRequest,
    AddOptionServerRequest,
    UpdateSettingRequest,
    PrivateNetworkApiAddServerPrivateNetworkRequest,
    PrivateNetworkApiSetServerPrivateNetworksRequest,
)


def unmarshal_CPU(data: Any) -> CPU:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'CPU' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("benchmark", None)
    args["benchmark"] = field

    field = data.get("core_count", None)
    args["core_count"] = field

    field = data.get("frequency", None)
    args["frequency"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("thread_count", None)
    args["thread_count"] = field

    return CPU(**args)


def unmarshal_Disk(data: Any) -> Disk:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Disk' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("capacity", None)
    args["capacity"] = field

    field = data.get("type", None)
    args["type_"] = field

    return Disk(**args)


def unmarshal_IP(data: Any) -> IP:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'IP' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("address", None)
    args["address"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("reverse", None)
    args["reverse"] = field

    field = data.get("reverse_status", None)
    args["reverse_status"] = field

    field = data.get("reverse_status_message", None)
    args["reverse_status_message"] = field

    field = data.get("version", None)
    args["version"] = field

    return IP(**args)


def unmarshal_Memory(data: Any) -> Memory:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Memory' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("capacity", None)
    args["capacity"] = field

    field = data.get("frequency", None)
    args["frequency"] = field

    field = data.get("is_ecc", None)
    args["is_ecc"] = field

    field = data.get("type", None)
    args["type_"] = field

    return Memory(**args)


def unmarshal_OSOSField(data: Any) -> OSOSField:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'OSOSField' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("default_value", None)
    args["default_value"] = field

    field = data.get("editable", None)
    args["editable"] = field

    field = data.get("required", None)
    args["required"] = field

    return OSOSField(**args)


def unmarshal_OfferOptionOffer(data: Any) -> OfferOptionOffer:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'OfferOptionOffer' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("enabled", None)
    args["enabled"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("manageable", None)
    args["manageable"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("os_id", None)
    args["os_id"] = field

    field = data.get("price", None)
    args["price"] = unmarshal_Money(field) if field is not None else None

    field = data.get("subscription_period", None)
    args["subscription_period"] = field

    return OfferOptionOffer(**args)


def unmarshal_PersistentMemory(data: Any) -> PersistentMemory:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'PersistentMemory' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("capacity", None)
    args["capacity"] = field

    field = data.get("frequency", None)
    args["frequency"] = field

    field = data.get("type", None)
    args["type_"] = field

    return PersistentMemory(**args)


def unmarshal_RaidController(data: Any) -> RaidController:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'RaidController' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("model", None)
    args["model"] = field

    field = data.get("raid_level", None)
    args["raid_level"] = field

    return RaidController(**args)


def unmarshal_ServerInstall(data: Any) -> ServerInstall:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ServerInstall' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("hostname", None)
    args["hostname"] = field

    field = data.get("os_id", None)
    args["os_id"] = field

    field = data.get("service_url", None)
    args["service_url"] = field

    field = data.get("service_user", None)
    args["service_user"] = field

    field = data.get("ssh_key_ids", None)
    args["ssh_key_ids"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("user", None)
    args["user"] = field

    return ServerInstall(**args)


def unmarshal_ServerOption(data: Any) -> ServerOption:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ServerOption' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("expires_at", None)
    args["expires_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("manageable", None)
    args["manageable"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("status", None)
    args["status"] = field

    return ServerOption(**args)


def unmarshal_ServerRescueServer(data: Any) -> ServerRescueServer:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ServerRescueServer' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("password", None)
    args["password"] = field

    field = data.get("user", None)
    args["user"] = field

    return ServerRescueServer(**args)


def unmarshal_OS(data: Any) -> OS:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'OS' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("allowed", None)
    args["allowed"] = field

    field = data.get("enabled", None)
    args["enabled"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("license_required", None)
    args["license_required"] = field

    field = data.get("logo_url", None)
    args["logo_url"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("password", None)
    args["password"] = unmarshal_OSOSField(field) if field is not None else None

    field = data.get("service_password", None)
    args["service_password"] = unmarshal_OSOSField(field) if field is not None else None

    field = data.get("service_user", None)
    args["service_user"] = unmarshal_OSOSField(field) if field is not None else None

    field = data.get("ssh", None)
    args["ssh"] = unmarshal_OSOSField(field) if field is not None else None

    field = data.get("user", None)
    args["user"] = unmarshal_OSOSField(field) if field is not None else None

    field = data.get("version", None)
    args["version"] = field

    return OS(**args)


def unmarshal_Offer(data: Any) -> Offer:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Offer' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("bandwidth", None)
    args["bandwidth"] = field

    field = data.get("commercial_range", None)
    args["commercial_range"] = field

    field = data.get("cpus", None)
    args["cpus"] = [unmarshal_CPU(v) for v in field] if field is not None else None

    field = data.get("disks", None)
    args["disks"] = [unmarshal_Disk(v) for v in field] if field is not None else None

    field = data.get("enable", None)
    args["enable"] = field

    field = data.get("fee", None)
    args["fee"] = unmarshal_Money(field) if field is not None else None

    field = data.get("id", None)
    args["id"] = field

    field = data.get("incompatible_os_ids", None)
    args["incompatible_os_ids"] = field

    field = data.get("memories", None)
    args["memories"] = (
        [unmarshal_Memory(v) for v in field] if field is not None else None
    )

    field = data.get("name", None)
    args["name"] = field

    field = data.get("operation_path", None)
    args["operation_path"] = field

    field = data.get("options", None)
    args["options"] = (
        [unmarshal_OfferOptionOffer(v) for v in field] if field is not None else None
    )

    field = data.get("persistent_memories", None)
    args["persistent_memories"] = (
        [unmarshal_PersistentMemory(v) for v in field] if field is not None else None
    )

    field = data.get("price_per_hour", None)
    args["price_per_hour"] = unmarshal_Money(field) if field is not None else None

    field = data.get("price_per_month", None)
    args["price_per_month"] = unmarshal_Money(field) if field is not None else None

    field = data.get("private_bandwidth", None)
    args["private_bandwidth"] = field

    field = data.get("quota_name", None)
    args["quota_name"] = field

    field = data.get("raid_controllers", None)
    args["raid_controllers"] = (
        [unmarshal_RaidController(v) for v in field] if field is not None else None
    )

    field = data.get("shared_bandwidth", None)
    args["shared_bandwidth"] = field

    field = data.get("stock", None)
    args["stock"] = field

    field = data.get("subscription_period", None)
    args["subscription_period"] = field

    field = data.get("tags", None)
    args["tags"] = field

    return Offer(**args)


def unmarshal_Option(data: Any) -> Option:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Option' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    args["id"] = field

    field = data.get("manageable", None)
    args["manageable"] = field

    field = data.get("name", None)
    args["name"] = field

    return Option(**args)


def unmarshal_Server(data: Any) -> Server:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Server' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("boot_type", None)
    args["boot_type"] = field

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description", None)
    args["description"] = field

    field = data.get("domain", None)
    args["domain"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("install", None)
    args["install"] = unmarshal_ServerInstall(field) if field is not None else None

    field = data.get("ips", None)
    args["ips"] = [unmarshal_IP(v) for v in field] if field is not None else None

    field = data.get("name", None)
    args["name"] = field

    field = data.get("offer_id", None)
    args["offer_id"] = field

    field = data.get("offer_name", None)
    args["offer_name"] = field

    field = data.get("options", None)
    args["options"] = (
        [unmarshal_ServerOption(v) for v in field] if field is not None else None
    )

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("ping_status", None)
    args["ping_status"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("rescue_server", None)
    args["rescue_server"] = (
        unmarshal_ServerRescueServer(field) if field is not None else None
    )

    field = data.get("status", None)
    args["status"] = field

    field = data.get("tags", None)
    args["tags"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone", None)
    args["zone"] = field

    return Server(**args)


def unmarshal_ServerEvent(data: Any) -> ServerEvent:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ServerEvent' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("action", None)
    args["action"] = field

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return ServerEvent(**args)


def unmarshal_ServerPrivateNetwork(data: Any) -> ServerPrivateNetwork:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ServerPrivateNetwork' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("private_network_id", None)
    args["private_network_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("server_id", None)
    args["server_id"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("vlan", None)
    args["vlan"] = field

    return ServerPrivateNetwork(**args)


def unmarshal_Setting(data: Any) -> Setting:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Setting' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("enabled", None)
    args["enabled"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("type", None)
    args["type_"] = field

    return Setting(**args)


def unmarshal_BMCAccess(data: Any) -> BMCAccess:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'BMCAccess' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("expires_at", None)
    args["expires_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("login", None)
    args["login"] = field

    field = data.get("password", None)
    args["password"] = field

    field = data.get("url", None)
    args["url"] = field

    return BMCAccess(**args)


def unmarshal_GetServerMetricsResponse(data: Any) -> GetServerMetricsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'GetServerMetricsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("pings", None)
    args["pings"] = unmarshal_TimeSeries(field) if field is not None else None

    return GetServerMetricsResponse(**args)


def unmarshal_ListOSResponse(data: Any) -> ListOSResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListOSResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("os", None)
    args["os"] = [unmarshal_OS(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListOSResponse(**args)


def unmarshal_ListOffersResponse(data: Any) -> ListOffersResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListOffersResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("offers", None)
    args["offers"] = [unmarshal_Offer(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListOffersResponse(**args)


def unmarshal_ListOptionsResponse(data: Any) -> ListOptionsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListOptionsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("options", None)
    args["options"] = (
        [unmarshal_Option(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListOptionsResponse(**args)


def unmarshal_ListServerEventsResponse(data: Any) -> ListServerEventsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListServerEventsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("events", None)
    args["events"] = (
        [unmarshal_ServerEvent(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListServerEventsResponse(**args)


def unmarshal_ListServerPrivateNetworksResponse(
    data: Any,
) -> ListServerPrivateNetworksResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListServerPrivateNetworksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("server_private_networks", None)
    args["server_private_networks"] = (
        [unmarshal_ServerPrivateNetwork(v) for v in field]
        if field is not None
        else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListServerPrivateNetworksResponse(**args)


def unmarshal_ListServersResponse(data: Any) -> ListServersResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListServersResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("servers", None)
    args["servers"] = (
        [unmarshal_Server(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListServersResponse(**args)


def unmarshal_ListSettingsResponse(data: Any) -> ListSettingsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListSettingsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("settings", None)
    args["settings"] = (
        [unmarshal_Setting(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListSettingsResponse(**args)


def unmarshal_SetServerPrivateNetworksResponse(
    data: Any,
) -> SetServerPrivateNetworksResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SetServerPrivateNetworksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("server_private_networks", None)
    args["server_private_networks"] = (
        [unmarshal_ServerPrivateNetwork(v) for v in field]
        if field is not None
        else None
    )

    return SetServerPrivateNetworksResponse(**args)


def marshal_CreateServerRequestInstall(
    request: CreateServerRequestInstall,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.hostname is not None:
        output["hostname"] = request.hostname

    if request.os_id is not None:
        output["os_id"] = request.os_id

    if request.password is not None:
        output["password"] = request.password

    if request.service_password is not None:
        output["service_password"] = request.service_password

    if request.service_user is not None:
        output["service_user"] = request.service_user

    if request.ssh_key_ids is not None:
        output["ssh_key_ids"] = request.ssh_key_ids

    if request.user is not None:
        output["user"] = request.user

    return output


def marshal_AddOptionServerRequest(
    request: AddOptionServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.expires_at is not None:
        output["expires_at"] = request.expires_at

    return output


def marshal_CreateServerRequest(
    request: CreateServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )

    if request.description is not None:
        output["description"] = request.description

    if request.install is not None:
        output["install"] = marshal_CreateServerRequestInstall(
            request.install, defaults
        )

    if request.name is not None:
        output["name"] = request.name

    if request.offer_id is not None:
        output["offer_id"] = request.offer_id

    if request.option_ids is not None:
        output["option_ids"] = request.option_ids

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_InstallServerRequest(
    request: InstallServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.hostname is not None:
        output["hostname"] = request.hostname

    if request.os_id is not None:
        output["os_id"] = request.os_id

    if request.password is not None:
        output["password"] = request.password

    if request.service_password is not None:
        output["service_password"] = request.service_password

    if request.service_user is not None:
        output["service_user"] = request.service_user

    if request.ssh_key_ids is not None:
        output["ssh_key_ids"] = request.ssh_key_ids

    if request.user is not None:
        output["user"] = request.user

    return output


def marshal_PrivateNetworkApiAddServerPrivateNetworkRequest(
    request: PrivateNetworkApiAddServerPrivateNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.private_network_id is not None:
        output["private_network_id"] = request.private_network_id

    return output


def marshal_PrivateNetworkApiSetServerPrivateNetworksRequest(
    request: PrivateNetworkApiSetServerPrivateNetworksRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.private_network_ids is not None:
        output["private_network_ids"] = request.private_network_ids

    return output


def marshal_RebootServerRequest(
    request: RebootServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.boot_type is not None:
        output["boot_type"] = ServerBootType(request.boot_type)

    return output


def marshal_StartBMCAccessRequest(
    request: StartBMCAccessRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.ip is not None:
        output["ip"] = request.ip

    return output


def marshal_StartServerRequest(
    request: StartServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.boot_type is not None:
        output["boot_type"] = ServerBootType(request.boot_type)

    return output


def marshal_UpdateIPRequest(
    request: UpdateIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.reverse is not None:
        output["reverse"] = request.reverse

    return output


def marshal_UpdateServerRequest(
    request: UpdateServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.description is not None:
        output["description"] = request.description

    if request.name is not None:
        output["name"] = request.name

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_UpdateSettingRequest(
    request: UpdateSettingRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.enabled is not None:
        output["enabled"] = request.enabled

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from dateutil import parser
from .types import (
    ImageVisibility,
    Image,
    ListImagesResponse,
    ListNamespacesResponse,
    ListTagsResponse,
    Namespace,
    Tag,
    CreateNamespaceRequest,
    UpdateNamespaceRequest,
    UpdateImageRequest,
)


def unmarshal_Image(data: Any) -> Image:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Image' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("namespace_id", None)
    args["namespace_id"] = field

    field = data.get("size", None)
    args["size"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("status_message", None)
    args["status_message"] = field

    field = data.get("tags", None)
    args["tags"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("visibility", None)
    args["visibility"] = field

    return Image(**args)


def unmarshal_Namespace(data: Any) -> Namespace:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Namespace' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description", None)
    args["description"] = field

    field = data.get("endpoint", None)
    args["endpoint"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("image_count", None)
    args["image_count"] = field

    field = data.get("is_public", None)
    args["is_public"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("region", None)
    args["region"] = field

    field = data.get("size", None)
    args["size"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("status_message", None)
    args["status_message"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Namespace(**args)


def unmarshal_Tag(data: Any) -> Tag:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Tag' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("digest", None)
    args["digest"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("image_id", None)
    args["image_id"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Tag(**args)


def unmarshal_ListImagesResponse(data: Any) -> ListImagesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListImagesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("images", None)
    args["images"] = [unmarshal_Image(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListImagesResponse(**args)


def unmarshal_ListNamespacesResponse(data: Any) -> ListNamespacesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListNamespacesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("namespaces", None)
    args["namespaces"] = (
        [unmarshal_Namespace(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListNamespacesResponse(**args)


def unmarshal_ListTagsResponse(data: Any) -> ListTagsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListTagsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("tags", None)
    args["tags"] = [unmarshal_Tag(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListTagsResponse(**args)


def marshal_CreateNamespaceRequest(
    request: CreateNamespaceRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )

    if request.description is not None:
        output["description"] = request.description

    if request.is_public is not None:
        output["is_public"] = request.is_public

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_UpdateImageRequest(
    request: UpdateImageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.visibility is not None:
        output["visibility"] = ImageVisibility(request.visibility)

    return output


def marshal_UpdateNamespaceRequest(
    request: UpdateNamespaceRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.description is not None:
        output["description"] = request.description

    if request.is_public is not None:
        output["is_public"] = request.is_public

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from dateutil import parser
from .types import (
    ListPrivateNetworksResponse,
    PrivateNetwork,
    CreatePrivateNetworkRequest,
    UpdatePrivateNetworkRequest,
)


def unmarshal_PrivateNetwork(data: Any) -> PrivateNetwork:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'PrivateNetwork' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("subnets", None)
    args["subnets"] = field

    field = data.get("tags", None)
    args["tags"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone", None)
    args["zone"] = field

    return PrivateNetwork(**args)


def unmarshal_ListPrivateNetworksResponse(data: Any) -> ListPrivateNetworksResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListPrivateNetworksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("private_networks", None)
    args["private_networks"] = (
        [unmarshal_PrivateNetwork(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListPrivateNetworksResponse(**args)


def marshal_CreatePrivateNetworkRequest(
    request: CreatePrivateNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.subnets is not None:
        output["subnets"] = request.subnets

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_UpdatePrivateNetworkRequest(
    request: UpdatePrivateNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.subnets is not None:
        output["subnets"] = request.subnets

    if request.tags is not None:
        output["tags"] = request.tags

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import ListImagesRequestOrderBy
from .types import ListLocalImagesRequestOrderBy
from .types import ListVersionsRequestOrderBy
from .types import LocalImageType
from .types import Category
from .types import Image
from .types import ListCategoriesResponse
from .types import ListImagesResponse
from .types import ListLocalImagesResponse
from .types import ListVersionsResponse
from .types import LocalImage
from .types import Version
from .api import MarketplaceV2API

__all__ = [
    "ListImagesRequestOrderBy",
    "ListLocalImagesRequestOrderBy",
    "ListVersionsRequestOrderBy",
    "LocalImageType",
    "Category",
    "Image",
    "ListCategoriesResponse",
    "ListImagesResponse",
    "ListLocalImagesResponse",
    "ListVersionsResponse",
    "LocalImage",
    "Version",
    "MarketplaceV2API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Zone,
)


class FlexibleIPStatus(str, Enum):
    UNKNOWN = "unknown"
    READY = "ready"
    UPDATING = "updating"
    ATTACHED = "attached"
    ERROR = "error"
    DETACHING = "detaching"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class ListFlexibleIPsRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class MACAddressStatus(str, Enum):
    UNKNOWN = "unknown"
    READY = "ready"
    UPDATING = "updating"
    USED = "used"
    ERROR = "error"
    DELETING = "deleting"

    def __str__(self) -> str:
        return str(self.value)


class MACAddressType(str, Enum):
    UNKNOWN_TYPE = "unknown_type"
    VMWARE = "vmware"
    XEN = "xen"
    KVM = "kvm"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class AttachFlexibleIPsResponse:
    """
    Attach flexible i ps response.
    """

    total_count: int
    """
    Total count of flexible IPs that are being updated.
    """

    flexible_ips: List[FlexibleIP]
    """
    List of flexible IPs in an updating state.
    """


@dataclass
class DetachFlexibleIPsResponse:
    """
    Detach flexible i ps response.
    """

    total_count: int
    """
    Total count of flexible IPs that are being detached.
    """

    flexible_ips: List[FlexibleIP]
    """
    List of flexible IPs in a detaching state.
    """


@dataclass
class FlexibleIP:
    """
    Flexible ip.
    """

    id: str
    """
    ID of the flexible IP.
    """

    organization_id: str
    """
    ID of the Organization the flexible IP is attached to.
    """

    project_id: str
    """
    ID of the Project the flexible IP is attached to.
    """

    description: str
    """
    Flexible IP description.
    """

    tags: List[str]
    """
    Flexible IP tags.
    """

    updated_at: Optional[datetime]
    """
    Date on which the flexible IP was last updated.
    """

    created_at: Optional[datetime]
    """
    Date on which the flexible IP was created.
    """

    status: FlexibleIPStatus
    """
    Flexible IP status.
    - ready : flexible IP is created and ready to be attached to a server or to be associated with a virtual MAC.
    - updating: flexible IP is being attached to a server or a virtual MAC operation is ongoing
    - attached: flexible IP is attached to a server
    - error: a flexible IP operation resulted in an error
    - detaching: flexible IP is being detached from a server
    - locked: the resource of the flexible IP is locked.
    """

    ip_address: str
    """
    IP of the flexible IP.
    """

    mac_address: Optional[MACAddress]
    """
    MAC address of the flexible IP.
    """

    server_id: Optional[str]
    """
    ID of the server linked to the flexible IP.
    """

    reverse: str
    """
    Reverse DNS value.
    """

    zone: Zone
    """
    Availability Zone of the flexible IP.
    """


@dataclass
class ListFlexibleIPsResponse:
    """
    List flexible i ps response.
    """

    total_count: int
    """
    Total count of matching flexible IPs.
    """

    flexible_ips: List[FlexibleIP]
    """
    List of all flexible IPs.
    """


@dataclass
class MACAddress:
    """
    Mac address.
    """

    id: str
    """
    ID of the flexible IP.
    """

    mac_address: str
    """
    MAC address of the Virtual MAC.
    """

    mac_type: MACAddressType
    """
    Type of virtual MAC.
    """

    status: MACAddressStatus
    """
    Status of virtual MAC.
    """

    updated_at: Optional[datetime]
    """
    Date on which the virtual MAC was last updated.
    """

    created_at: Optional[datetime]
    """
    Date on which the virtual MAC was created.
    """

    zone: Zone
    """
    MAC address IP Availability Zone.
    """


@dataclass
class CreateFlexibleIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    ID of the project to associate with the Flexible IP.
    """

    description: str
    """
    Flexible IP description (max. of 255 characters).
    """

    tags: Optional[List[str]]
    """
    Tags to associate to the flexible IP.
    """

    server_id: Optional[str]
    """
    ID of the server to which the newly created flexible IP will be attached.
    """

    reverse: Optional[str]
    """
    Value of the reverse DNS.
    """

    is_ipv6: bool
    """
    Defines whether the flexible IP has an IPv6 address.
    """


@dataclass
class GetFlexibleIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fip_id: str
    """
    ID of the flexible IP.
    """


@dataclass
class ListFlexibleIPsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListFlexibleIPsRequestOrderBy]
    """
    Sort order of the returned flexible IPs.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Maximum number of flexible IPs per page.
    """

    tags: Optional[List[str]]
    """
    Filter by tag, only flexible IPs with one or more matching tags will be returned.
    """

    status: Optional[List[FlexibleIPStatus]]
    """
    Filter by status, only flexible IPs with this status will be returned.
    """

    server_ids: Optional[List[str]]
    """
    Filter by server IDs, only flexible IPs with these server IDs will be returned.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID, only flexible IPs from this Organization will be returned.
    """

    project_id: Optional[str]
    """
    Filter by Project ID, only flexible IPs from this Project will be returned.
    """


@dataclass
class UpdateFlexibleIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fip_id: str
    """
    ID of the flexible IP to update.
    """

    description: Optional[str]
    """
    Flexible IP description (max. 255 characters).
    """

    tags: Optional[List[str]]
    """
    Tags associated with the flexible IP.
    """

    reverse: Optional[str]
    """
    Value of the reverse DNS.
    """


@dataclass
class DeleteFlexibleIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fip_id: str
    """
    ID of the flexible IP to delete.
    """


@dataclass
class AttachFlexibleIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fips_ids: List[str]
    """
    List of flexible IP IDs to attach to a server.
    Multiple IDs can be provided, but note that flexible IPs must belong to the same MAC group (see details about MAC groups).
    """

    server_id: str
    """
    ID of the server on which to attach the flexible IPs.
    """


@dataclass
class DetachFlexibleIPRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fips_ids: List[str]
    """
    List of flexible IP IDs to detach from a server. Multiple IDs can be provided. Note that flexible IPs must belong to the same MAC group.
    """


@dataclass
class GenerateMACAddrRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fip_id: str
    """
    ID of the flexible IP for which to generate a virtual MAC.
    """

    mac_type: Optional[MACAddressType]
    """
    TODO.
    """


@dataclass
class DuplicateMACAddrRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fip_id: str
    """
    ID of the flexible IP on which to duplicate the virtual MAC.
    Note that the flexible IPs need to be attached to the same server.
    """

    duplicate_from_fip_id: str
    """
    ID of the flexible IP to duplicate the Virtual MAC from.
    Note that flexible IPs need to be attached to the same server.
    """


@dataclass
class MoveMACAddrRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fip_id: str

    dst_fip_id: str


@dataclass
class DeleteMACAddrRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    fip_id: str
    """
    ID of the flexible IP from which to delete the virtual MAC.
    If the flexible IP belongs to a MAC group, the MAC will be removed from both the MAC group and flexible IP.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from dateutil import parser
from .types import (
    EyeColors,
    Human,
    ListHumansResponse,
    RegisterResponse,
    RegisterRequest,
    CreateHumanRequest,
    UpdateHumanRequest,
)


def unmarshal_Human(data: Any) -> Human:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Human' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("altitude_in_meter", None)
    args["altitude_in_meter"] = field

    field = data.get("altitude_in_millimeter", None)
    args["altitude_in_millimeter"] = field

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("eyes_color", None)
    args["eyes_color"] = field

    field = data.get("fingers_count", None)
    args["fingers_count"] = field

    field = data.get("hair_count", None)
    args["hair_count"] = field

    field = data.get("height", None)
    args["height"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("is_happy", None)
    args["is_happy"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("shoe_size", None)
    args["shoe_size"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Human(**args)


def unmarshal_ListHumansResponse(data: Any) -> ListHumansResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListHumansResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("humans", None)
    args["humans"] = [unmarshal_Human(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListHumansResponse(**args)


def unmarshal_RegisterResponse(data: Any) -> RegisterResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'RegisterResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("access_key", None)
    args["access_key"] = field

    field = data.get("secret_key", None)
    args["secret_key"] = field

    return RegisterResponse(**args)


def marshal_CreateHumanRequest(
    request: CreateHumanRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )

    if request.altitude_in_meter is not None:
        output["altitude_in_meter"] = request.altitude_in_meter

    if request.altitude_in_millimeter is not None:
        output["altitude_in_millimeter"] = request.altitude_in_millimeter

    if request.eyes_color is not None:
        output["eyes_color"] = EyeColors(request.eyes_color)

    if request.fingers_count is not None:
        output["fingers_count"] = request.fingers_count

    if request.hair_count is not None:
        output["hair_count"] = request.hair_count

    if request.height is not None:
        output["height"] = request.height

    if request.is_happy is not None:
        output["is_happy"] = request.is_happy

    if request.name is not None:
        output["name"] = request.name

    if request.shoe_size is not None:
        output["shoe_size"] = request.shoe_size

    return output


def marshal_RegisterRequest(
    request: RegisterRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.username is not None:
        output["username"] = request.username

    return output


def marshal_UpdateHumanRequest(
    request: UpdateHumanRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.altitude_in_meter is not None:
        output["altitude_in_meter"] = request.altitude_in_meter

    if request.altitude_in_millimeter is not None:
        output["altitude_in_millimeter"] = request.altitude_in_millimeter

    if request.eyes_color is not None:
        output["eyes_color"] = EyeColors(request.eyes_color)

    if request.fingers_count is not None:
        output["fingers_count"] = request.fingers_count

    if request.hair_count is not None:
        output["hair_count"] = request.hair_count

    if request.height is not None:
        output["height"] = request.height

    if request.is_happy is not None:
        output["is_happy"] = request.is_happy

    if request.name is not None:
        output["name"] = request.name

    if request.shoe_size is not None:
        output["shoe_size"] = request.shoe_size

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from dateutil import parser
from .types import (
    CreateEmailRequestAddress,
    CreateEmailRequestAttachment,
    CreateEmailResponse,
    Domain,
    DomainLastStatus,
    DomainLastStatusDkimRecord,
    DomainLastStatusSpfRecord,
    DomainStatistics,
    Email,
    EmailTry,
    ListDomainsResponse,
    ListEmailsResponse,
    Statistics,
    CreateEmailRequest,
    CreateDomainRequest,
)


def unmarshal_DomainStatistics(data: Any) -> DomainStatistics:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DomainStatistics' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("canceled_count", None)
    args["canceled_count"] = field

    field = data.get("failed_count", None)
    args["failed_count"] = field

    field = data.get("sent_count", None)
    args["sent_count"] = field

    field = data.get("total_count", None)
    args["total_count"] = field

    return DomainStatistics(**args)


def unmarshal_EmailTry(data: Any) -> EmailTry:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'EmailTry' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("code", None)
    args["code"] = field

    field = data.get("message", None)
    args["message"] = field

    field = data.get("rank", None)
    args["rank"] = field

    field = data.get("tried_at", None)
    args["tried_at"] = parser.isoparse(field) if type(field) is str else field

    return EmailTry(**args)


def unmarshal_Domain(data: Any) -> Domain:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Domain' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("dkim_config", None)
    args["dkim_config"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("last_error", None)
    args["last_error"] = field

    field = data.get("last_valid_at", None)
    args["last_valid_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("next_check_at", None)
    args["next_check_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("region", None)
    args["region"] = field

    field = data.get("revoked_at", None)
    args["revoked_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("spf_config", None)
    args["spf_config"] = field

    field = data.get("statistics", None)
    args["statistics"] = (
        unmarshal_DomainStatistics(field) if field is not None else None
    )

    field = data.get("status", None)
    args["status"] = field

    return Domain(**args)


def unmarshal_DomainLastStatusDkimRecord(data: Any) -> DomainLastStatusDkimRecord:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DomainLastStatusDkimRecord' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("error", None)
    args["error"] = field

    field = data.get("last_valid_at", None)
    args["last_valid_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("status", None)
    args["status"] = field

    return DomainLastStatusDkimRecord(**args)


def unmarshal_DomainLastStatusSpfRecord(data: Any) -> DomainLastStatusSpfRecord:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DomainLastStatusSpfRecord' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("error", None)
    args["error"] = field

    field = data.get("last_valid_at", None)
    args["last_valid_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("status", None)
    args["status"] = field

    return DomainLastStatusSpfRecord(**args)


def unmarshal_Email(data: Any) -> Email:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Email' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("flags", None)
    args["flags"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("last_tries", None)
    args["last_tries"] = (
        [unmarshal_EmailTry(v) for v in field] if field is not None else None
    )

    field = data.get("mail_from", None)
    args["mail_from"] = field

    field = data.get("mail_rcpt", None)
    args["mail_rcpt"] = field

    field = data.get("message_id", None)
    args["message_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("rcpt_to", None)
    args["rcpt_to"] = field

    field = data.get("rcpt_type", None)
    args["rcpt_type"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("status_details", None)
    args["status_details"] = field

    field = data.get("subject", None)
    args["subject"] = field

    field = data.get("try_count", None)
    args["try_count"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Email(**args)


def unmarshal_CreateEmailResponse(data: Any) -> CreateEmailResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'CreateEmailResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("emails", None)
    args["emails"] = [unmarshal_Email(v) for v in field] if field is not None else None

    return CreateEmailResponse(**args)


def unmarshal_DomainLastStatus(data: Any) -> DomainLastStatus:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DomainLastStatus' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("dkim_record", None)
    args["dkim_record"] = (
        unmarshal_DomainLastStatusDkimRecord(field) if field is not None else None
    )

    field = data.get("domain_id", None)
    args["domain_id"] = field

    field = data.get("domain_name", None)
    args["domain_name"] = field

    field = data.get("spf_record", None)
    args["spf_record"] = (
        unmarshal_DomainLastStatusSpfRecord(field) if field is not None else None
    )

    return DomainLastStatus(**args)


def unmarshal_ListDomainsResponse(data: Any) -> ListDomainsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("domains", None)
    args["domains"] = (
        [unmarshal_Domain(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListDomainsResponse(**args)


def unmarshal_ListEmailsResponse(data: Any) -> ListEmailsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListEmailsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("emails", None)
    args["emails"] = [unmarshal_Email(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListEmailsResponse(**args)


def unmarshal_Statistics(data: Any) -> Statistics:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Statistics' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("canceled_count", None)
    args["canceled_count"] = field

    field = data.get("failed_count", None)
    args["failed_count"] = field

    field = data.get("new_count", None)
    args["new_count"] = field

    field = data.get("sending_count", None)
    args["sending_count"] = field

    field = data.get("sent_count", None)
    args["sent_count"] = field

    field = data.get("total_count", None)
    args["total_count"] = field

    return Statistics(**args)


def marshal_CreateEmailRequestAddress(
    request: CreateEmailRequestAddress,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.email is not None:
        output["email"] = request.email

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_CreateEmailRequestAttachment(
    request: CreateEmailRequestAttachment,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.content is not None:
        output["content"] = request.content

    if request.name is not None:
        output["name"] = request.name

    if request.type_ is not None:
        output["type"] = request.type_

    return output


def marshal_CreateDomainRequest(
    request: CreateDomainRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.accept_tos is not None:
        output["accept_tos"] = request.accept_tos

    if request.domain_name is not None:
        output["domain_name"] = request.domain_name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_CreateEmailRequest(
    request: CreateEmailRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.attachments is not None:
        output["attachments"] = [
            marshal_CreateEmailRequestAttachment(v, defaults)
            for v in request.attachments
        ]

    if request.bcc is not None:
        output["bcc"] = [
            marshal_CreateEmailRequestAddress(v, defaults) for v in request.bcc
        ]

    if request.cc is not None:
        output["cc"] = [
            marshal_CreateEmailRequestAddress(v, defaults) for v in request.cc
        ]

    if request.from_ is not None:
        output["from"] = marshal_CreateEmailRequestAddress(request.from_, defaults)

    if request.html is not None:
        output["html"] = request.html

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.send_before is not None:
        output["send_before"] = request.send_before

    if request.subject is not None:
        output["subject"] = request.subject

    if request.text is not None:
        output["text"] = request.text

    if request.to is not None:
        output["to"] = [
            marshal_CreateEmailRequestAddress(v, defaults) for v in request.to
        ]

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from dateutil import parser
from .types import (
    AddSubnetsResponse,
    DeleteSubnetsResponse,
    ListPrivateNetworksResponse,
    ListVPCsResponse,
    PrivateNetwork,
    SetSubnetsResponse,
    Subnet,
    VPC,
    CreateVPCRequest,
    UpdateVPCRequest,
    CreatePrivateNetworkRequest,
    UpdatePrivateNetworkRequest,
    MigrateZonalPrivateNetworksRequest,
    SetSubnetsRequest,
    AddSubnetsRequest,
    DeleteSubnetsRequest,
)


def unmarshal_Subnet(data: Any) -> Subnet:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Subnet' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("subnet", None)
    args["subnet"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Subnet(**args)


def unmarshal_PrivateNetwork(data: Any) -> PrivateNetwork:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'PrivateNetwork' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("dhcp_enabled", None)
    args["dhcp_enabled"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("region", None)
    args["region"] = field

    field = data.get("subnets", None)
    args["subnets"] = (
        [unmarshal_Subnet(v) for v in field] if field is not None else None
    )

    field = data.get("tags", None)
    args["tags"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("vpc_id", None)
    args["vpc_id"] = field

    return PrivateNetwork(**args)


def unmarshal_VPC(data: Any) -> VPC:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'VPC' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("is_default", None)
    args["is_default"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("private_network_count", None)
    args["private_network_count"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("region", None)
    args["region"] = field

    field = data.get("tags", None)
    args["tags"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return VPC(**args)


def unmarshal_AddSubnetsResponse(data: Any) -> AddSubnetsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'AddSubnetsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("subnets", None)
    args["subnets"] = field

    return AddSubnetsResponse(**args)


def unmarshal_DeleteSubnetsResponse(data: Any) -> DeleteSubnetsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DeleteSubnetsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("subnets", None)
    args["subnets"] = field

    return DeleteSubnetsResponse(**args)


def unmarshal_ListPrivateNetworksResponse(data: Any) -> ListPrivateNetworksResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListPrivateNetworksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("private_networks", None)
    args["private_networks"] = (
        [unmarshal_PrivateNetwork(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListPrivateNetworksResponse(**args)


def unmarshal_ListVPCsResponse(data: Any) -> ListVPCsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListVPCsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    args["total_count"] = field

    field = data.get("vpcs", None)
    args["vpcs"] = [unmarshal_VPC(v) for v in field] if field is not None else None

    return ListVPCsResponse(**args)


def unmarshal_SetSubnetsResponse(data: Any) -> SetSubnetsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SetSubnetsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("subnets", None)
    args["subnets"] = field

    return SetSubnetsResponse(**args)


def marshal_AddSubnetsRequest(
    request: AddSubnetsRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.subnets is not None:
        output["subnets"] = request.subnets

    return output


def marshal_CreatePrivateNetworkRequest(
    request: CreatePrivateNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.subnets is not None:
        output["subnets"] = request.subnets

    if request.tags is not None:
        output["tags"] = request.tags

    if request.vpc_id is not None:
        output["vpc_id"] = request.vpc_id

    return output


def marshal_CreateVPCRequest(
    request: CreateVPCRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_DeleteSubnetsRequest(
    request: DeleteSubnetsRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.subnets is not None:
        output["subnets"] = request.subnets

    return output


def marshal_MigrateZonalPrivateNetworksRequest(
    request: MigrateZonalPrivateNetworksRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )

    if request.private_network_ids is not None:
        output["private_network_ids"] = request.private_network_ids

    return output


def marshal_SetSubnetsRequest(
    request: SetSubnetsRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.subnets is not None:
        output["subnets"] = request.subnets

    return output


def marshal_UpdatePrivateNetworkRequest(
    request: UpdatePrivateNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_UpdateVPCRequest(
    request: UpdateVPCRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.tags is not None:
        output["tags"] = request.tags

    return output
